# -*- coding: utf-8 -*-

#    Copyright 2016 Fabien Bourgeois <fabien@yaltik.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
from datetime import datetime
import argparse
import odoorpc


class Import(object):

    @classmethod
    def odoo_connect(cls, dbname, password):
        """ Odoo connection """
        cls.odoo = odoorpc.ODOO('127.0.0.1', port=8069)
        cls.odoo.config['timeout'] = 600
        cls.odoo.login(dbname, 'admin', password)

    @classmethod
    def init(cls):
        """ Populates database with needed objects """
        SeasonObj = cls.odoo.env['golem.season']
        seasons = SeasonObj.search([])
        if len(seasons) < 1:
            cls.s2015 = SeasonObj.create({'name': u'2015/2016',
                                          'date_start': '2015-09-01',
                                          'date_end': '2016-08-31'})
            cls.s2016 = SeasonObj.create({'name': u'2016/2017',
                                          'date_start': '2016-09-01',
                                          'date_end': '2017-08-31'})
            SeasonObj.browse([cls.s2016]).do_default_season()
        else:
            d2015 = [('date_start', '=', '2015-09-01')]
            d2016 = [('date_start', '=', '2016-09-01')]
            cls.s2015 = SeasonObj.search(d2015)[0]
            cls.s2016 = SeasonObj.search(d2016)[0]
        with open('./all.json') as f:
            cls.data = json.load(f)

        def schema_from_2015(schema):
            return [r for r in cls.data if r['doc']['schema'] == schema and
                    r['doc']['season'] == 2015]

        cls.m2015 = schema_from_2015('member')
        cls.a2015 = schema_from_2015('activity')
        dfr = [('code', '=', 'FR')]
        cls.FR = cls.odoo.env['res.country'].search(dfr)[0]

    @classmethod
    def members2015(cls):
        """ Base member import : season 2015 only """
        MemberObj = cls.odoo.env['golem.member']
        known_numbers = []
        for m in cls.m2015:
            # TODO: and not done --> tags, skills
            auth = m['doc']['authorizations']
            comm = m['doc']['communicationModes']
            mail, phone, mobile = '', '', ''
            if m['doc']['mails']:
                mail = m['doc']['mails'][0]['value']
            if m['doc']['tels']:
                tel1 = m['doc']['tels'][0]['value']
                if len(m['doc']['tels']) == 1:
                    phone = tel1
                else:
                    tel2 = m['doc']['tels'][1]['value']
                    if tel1.startswith('06') or tel1.startswith('07'):
                        mobile = tel1
                        phone = tel2
                    else:
                        phone = tel1
                        mobile = tel2
            birthdate = None
            if m['doc']['birthday']:
                birthdate = datetime.strptime(m['doc']['birthday'][:15],
                                              '%a %b %d %Y')
                birthdate = birthdate.strftime('%Y-%m-%d')
            gender = None
            if m['doc']['gender'] == 'm':
                gender = 'male'
            elif m['doc']['gender'] == 'f':
                gender = 'female'
            nationality = None
            if m['doc']['nationality'] == u'française':
                nationality = cls.FR
            member = {'golemv1_key': m['key'],
                      'season_ids': [(6, 0, [cls.s2015])],
                      'lastname': m['doc']['lastname'],
                      'firstname': m['doc']['firstname'],
                      'street': m['doc']['address'],
                      'zip': m['doc']['postalCode'],
                      'city': m['doc']['city'],
                      'comment': m['doc']['note'],
                      'number_manual': m['doc']['number'],
                      'birthdate_date': birthdate,
                      'function': m['doc']['profession'],
                      'gender': gender,
                      'nationality_id': nationality,
                      'activities_participation': auth['activities'],
                      'pictures_agreement': auth['photos'],
                      'opt_out': not comm['mail'],
                      'opt_out_sms': not comm['tel'],
                      'email': mail,
                      'phone': phone,
                      'mobile': mobile,
                      }
            if m['doc']['number'] in known_numbers:
                member['number_manual'] = m['doc']['number'] + 'DUP'
            else:
                known_numbers.append(m['doc']['number'])
            MemberObj.create(member)

    @classmethod
    def _partner_creation(cls, firstname, lastname):
        """ Helper to handle partner creation or linking """
        PartnerObj = cls.odoo.env['res.partner']
        dmn = ['&',
               ('firstname', '=', firstname),
               ('lastname', '=', lastname)]
        partner = PartnerObj.search(dmn)
        pid = None
        if partner:
            pid = partner[0]
        else:
            pid = PartnerObj.create({'is_company': False,
                                     'lastname': lastname,
                                     'firstname': firstname})
        return pid

    @classmethod
    def members2015_guardians(cls):
        """ Handle legal guardians """
        MemberObj = cls.odoo.env['golem.member']
        for m in cls.m2015:
            if m['doc']['guardianFirstname']:
                firstname = m['doc']['guardianFirstname']
                lastname = m['doc']['guardianLastname']
                gid = cls._partner_creation(firstname, lastname)
                dmn = ['&',
                       ('firstname', '=', m['doc']['firstname']),
                       ('lastname', '=', m['doc']['lastname'])]
                member = MemberObj.browse([MemberObj.search(dmn)[0]])[0]
                member.write({'legal_guardian_ids': [(6, 0, [gid])]})

    @classmethod
    def _master_activity(cls, a):
        """ For activities import : master activity creation or id return """
        ActivityObj = cls.odoo.env['golem.activity']
        default_categ_id = cls.odoo.env.ref(
            'golem_activity.golem_product_category_activities').id
        ma_code = a['doc']['code'][0:2]
        ma_name = a['doc']['label'].split(' ')[0]
        master = ActivityObj.search([('default_code', '=', ma_code)])
        if master:
            return master[0]
        else:
            return ActivityObj.create({'season_id': cls.s2015,
                                       'categ_id': default_categ_id,
                                       'default_code': ma_code,
                                       'name': ma_name})

    @classmethod
    def _animator(cls, a):
        """ For activities import : animator creation / linking """
        aid = None
        if a['doc']['monitor']:
            animator = a['doc']['monitor'].split(' ')
            a_firstname = animator[0]
            a_lastname = animator[1]
            aid = cls._partner_creation(a_firstname, a_lastname)
        return aid

    @classmethod
    def activities2015(cls):
        """ Base activity import for season 2015 """
        ActivitySessionObj = cls.odoo.env['golem.activity.session']
        # All activities import, and animator creation / linking
        for a in cls.a2015:
            descr = u'{}\n\n{}'.format(a['doc']['timeSlot'], a['doc']['note'])
            maid = cls._master_activity(a)
            aid = cls._animator(a)
            session = {'golemv1_key': a['key'],
                       'activity_id': maid,
                       'name': a['doc']['label'],
                       'default_code': a['doc']['code'],
                       'places': a['doc']['places'] or 99,
                       'animator_id': aid,
                       'description': descr}
            ActivitySessionObj.create(session)

    @classmethod
    def registrations2015(cls):
        """ Base activities registrations import for season 2015 """
        MemberObj = cls.odoo.env['golem.member']
        SessionObj = cls.odoo.env['golem.activity.session']
        for m in cls.m2015:
            registrations = m['doc']['activities']
            if len(registrations) > 0:
                for r in registrations:
                    session = SessionObj.search([('golemv1_key', '=', r)])
                    if session:
                        mk = m['key']
                        member = MemberObj.search([('golemv1_key', '=', mk)])
                        if member:
                            member = MemberObj.browse([member[0]])
                            sid = session[0]
                            member.write({'activity_session_ids': [(4, sid)]})

    @classmethod
    def all(cls, database, password, task):
        """ Run base import """
        cls.odoo_connect(database, password)
        cls.init()
        if task == 'members':
            cls.members2015()
            cls.members2015_guardians()
        elif task == 'activities':
            cls.activities2015()
            cls.registrations2015()
        else:
            cls.members2015()
            cls.members2015_guardians()
            cls.activities2015()
            cls.registrations2015()


def main():
    """ Main function with argument parsing """
    parser = argparse.ArgumentParser(description='Valentigney data import')
    parser.add_argument('-d', '--database', type=str, required=True,
                        help='database name')
    parser.add_argument('-p', '--password', type=str, required=True,
                        help='database password')
    parser.add_argument('-t', '--task', type=str,
                        help='optional import task (default:all)')
    args = parser.parse_args()
    Import.all(args.database, args.password, args.task)

if __name__ == "__main__":
    main()
